/*
 * Copyright 2012-2013 Denis Neuling 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */
package com.apitrary.api.client.util;

import java.util.logging.Logger;

import com.apitrary.api.annotation.Normalized;
import com.apitrary.api.response.Response;
import com.apitrary.api.response.normalized.Normalizer;

/**
 * <p>
 * NormalizationUtil class.
 * </p>
 *
 * @author Denis Neuling (denisneuling@gmail.com)
 *
 */
public class NormalizationUtil {
	private static Logger log = Logger.getLogger(NormalizationUtil.class.getName());

	/**
	 * <p>
	 * getNormalizer.
	 * </p>
	 *
	 * @param response
	 *            a {@link com.apitrary.api.response.Response} object.
	 * @param <T>
	 *            a T object.
	 * @return a {@link com.apitrary.api.response.normalized.Normalizer} object.
	 */
	@SuppressWarnings("unchecked")
	public static <T> Normalizer getNormalizer(Response<T> response) {
		Normalizer normalizer = new Normalizer();
		Class<Normalizer> normalizerClazz = ClassUtil.getClassAnnotationValue(response.getClass(), Normalized.class, "value", Class.class);
		if (normalizerClazz != null) {
			try {
				normalizer = normalizerClazz.newInstance();
			} catch (InstantiationException e) {
				log.warning(e.getClass().getSimpleName() + ": " + e.getMessage());
			} catch (IllegalAccessException e) {
				log.warning(e.getClass().getSimpleName() + ": " + e.getMessage());
			}
		}
		return normalizer;
	}
}
